@@ -63,6 +63,16 @@ import multiprocessing
 __TOTAL_CPU = multiprocessing.cpu_count()
 __N_CPU = 1 if __TOTAL_CPU < 2 else __TOTAL_CPU - 1
 
+def __inner_loop_single_iteration(bessel_zeros_jj, bessel_jm_jj, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz):
+    return numpy.dot(besselj(0, bessel_zeros_nz * bessel_zeros_jj / bessel_zeros_n) / (bessel_jn * bessel_jm_jj),
+                     f_nz)
+
+__v_inner_loop = numpy.vectorize(__inner_loop_single_iteration)
+__v_inner_loop.excluded.add(2)
+__v_inner_loop.excluded.add(3)
+__v_inner_loop.excluded.add(4)
+__v_inner_loop.excluded.add(5)
+
 def hankel_transform(wavefield, max_radius, bessel_zeros, n_zeros=0, multipool=True):
     n = len(wavefield)
     if n_zeros <= 0: n_zeros = n
@@ -72,21 +82,21 @@ def hankel_transform(wavefield, max_radius, bessel_zeros, n_zeros=0, multipool=T
     max_frequency = bessel_zeros[n] / (2 * numpy.pi * max_radius)
     m2 = m1 * max_radius / max_frequency
     f = numpy.divide(wavefield, m1)
-    bessel_jn = numpy.abs(besselj(1, bessel_zeros[:n_zeros])) / (2 / bessel_zeros[n])
+
+    bessel_zeros_nz = bessel_zeros[0:n_zeros]
+    bessel_zeros_n  = bessel_zeros[n]
+    f_nz            = f[0:n_zeros]
+    bessel_jn = numpy.abs(besselj(1, bessel_zeros_nz)) / (2 / bessel_zeros_n)
     bessel_jm = numpy.abs(besselj(1, bessel_zeros))
 
     if multipool:
-        args = __create_args(__N_CPU, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n)
+        args = __create_args(__N_CPU, bessel_zeros[0:n], bessel_jm[0:n], bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz, n)
 
         p = multiprocessing.Pool(__N_CPU)
         result = numpy.concatenate(p.map(__calculate, args))
         p.close()
     else:
-        result = numpy.full(n, 0j)
-
-        for jj in range(0, n):
-            c = besselj(0, bessel_zeros[:n_zeros] * bessel_zeros[jj] / bessel_zeros[n]) / (bessel_jn * bessel_jm[jj])
-            result[jj] = numpy.dot(c[:n_zeros], f[:n_zeros])
+        result = __v_inner_loop(bessel_zeros[0:n], bessel_jm[0:n], bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz)
 
     result = result.conjugate().T
     result = numpy.multiply(result, m2)
@@ -94,28 +104,21 @@ def hankel_transform(wavefield, max_radius, bessel_zeros, n_zeros=0, multipool=T
 
     return result
 
-
 def __calculate(args):
-    index_i, index_f, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n = args
+    index_i, index_f, bessel_zeros, bessel_jm, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz = args
 
-    result = numpy.full((index_f-index_i), 0j)
+    return __v_inner_loop(bessel_zeros[index_i:index_f], bessel_jm[index_i:index_f], bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz)
 
-    for jj in range(index_i, index_f):
-        c = besselj(0, bessel_zeros[:n_zeros] * bessel_zeros[jj] / bessel_zeros[n]) / (bessel_jn * bessel_jm[jj])
-        result[jj-index_i] = numpy.dot(c[:n_zeros], f[:n_zeros])
-
-    return result
-
-def __create_args(n_pools, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n):
+def __create_args(n_pools, bessel_zeros, bessel_jm, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz, n):
     args = [None]*n_pools
 
     n_el = int(n/n_pools)
     if n % n_pools == 0:
         for i in range(n_pools):
-            args[i] = i * n_el, (i+1) * n_el, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n
+            args[i] = i * n_el, (i+1) * n_el, bessel_zeros, bessel_jm, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz
     else:
         for i in range(n_pools - 1):
-            args[i] = i * n_el, (i+1) * n_el, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n
-        args[n_pools-1] = (n_pools - 1) * n_el, n_pools*n_el + n % n_pools, bessel_zeros, bessel_jn, bessel_jm, f, n_zeros, n
+            args[i] = i * n_el, (i+1) * n_el, bessel_zeros, bessel_jm, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz
+        args[n_pools-1] = (n_pools - 1) * n_el, n_pools * n_el + n % n_pools, bessel_zeros, bessel_jm, bessel_zeros_nz, bessel_zeros_n, bessel_jn, f_nz
 
     return args
